@@ -114,10 +114,10 @@
 #define NO_TIME_PROVIDED 0 /* indicate we do not provide any cached time */
 
 /* forward definitions */
-static rsRetVal processBatchMain(void *pVoid, batch_t *pBatch, wti_t *pWti);
-static rsRetVal doSubmitToActionQ(action_t *pAction, wti_t *pWti, msg_t*);
-static rsRetVal doSubmitToActionQComplex(action_t *pAction, wti_t *pWti, msg_t*);
-static rsRetVal doSubmitToActionQNotAllMark(action_t *pAction, wti_t *pWti, msg_t*);
+static rsRetVal processBatchMain(void *pVoid, batch_t *pBatch, wti_t * const pWti);
+static rsRetVal doSubmitToActionQ(action_t * const pAction, wti_t * const pWti, msg_t*);
+static rsRetVal doSubmitToActionQComplex(action_t * const pAction, wti_t * const pWti, msg_t*);
+static rsRetVal doSubmitToActionQNotAllMark(action_t * const pAction, wti_t * const pWti, msg_t*);
 
 /* object static data (once for all instances) */
 DEFobjCurrIf(obj)
@@ -172,6 +172,8 @@ configSettings_t cs_save;				/* our saved (scope!) config settings */
  * is no better name available.
  */
 int iActionNbr = 0;
+int bActionReportSuspension = 1;
+int bActionReportSuspensionCont = 0;
 
 /* tables for interfacing with the v6 config system */
 static struct cnfparamdescr cnfparamdescr[] = {
@@ -184,6 +186,8 @@ static struct cnfparamdescr cnfparamdescr[] = {
 	{ "action.execonlywhenpreviousissuspended", eCmdHdlrBinary, 0 }, /* legacy: actionexeconlywhenpreviousissuspended */
 	{ "action.repeatedmsgcontainsoriginalmsg", eCmdHdlrBinary, 0 }, /* legacy: repeatedmsgcontainsoriginalmsg */
 	{ "action.resumeretrycount", eCmdHdlrInt, 0 }, /* legacy: actionresumeretrycount */
+	{ "action.reportsuspension", eCmdHdlrBinary, 0 },
+	{ "action.reportsuspensioncontinuation", eCmdHdlrBinary, 0 },
 	{ "action.resumeinterval", eCmdHdlrInt, 0 }
 };
 static struct cnfparamblk pblk =
@@ -217,7 +221,7 @@ static struct cnfparamblk pblk =
  * a lot. So we simply return the system time.
  */
 static inline time_t
-getActNow(action_t *pThis)
+getActNow(action_t * const pThis)
 {
 	assert(pThis != NULL);
 	if(pThis->tActNow == -1) {
@@ -277,7 +281,7 @@ actionResetQueueParams(void)
 /* destructs an action descriptor object
  * rgerhards, 2007-08-01
  */
-rsRetVal actionDestruct(action_t *pThis)
+rsRetVal actionDestruct(action_t * const pThis)
 {
 	DEFiRet;
 	ASSERT(pThis != NULL);
@@ -310,6 +314,19 @@ finalize_it:
 }
 
 
+/* Disable action, this means it will never again be usable
+ * until rsyslog is reloaded. Use only as a last resort, but
+ * depends on output module.
+ * rgerhards, 2007-08-02
+ */
+static inline void
+actionDisable(action_t *__restrict__ const pThis)
+{
+	pThis->bDisabled = 1;
+}
+
+
+
 /* create a new action descriptor object
  * rgerhards, 2007-08-01
  * Note that it is vital to set proper initial values as the v6 config
@@ -333,6 +350,9 @@ rsRetVal actionConstruct(action_t **ppThis)
 	pThis->bExecWhenPrevSusp = 0;
 	pThis->bRepMsgHasMsg = 0;
 	pThis->bDisabled = 0;
+	pThis->isTransactional = 0;
+	pThis->bReportSuspension = -1; /* indicate "not yet set" */
+	pThis->bReportSuspensionCont = -1; /* indicate "not yet set" */
 	pThis->tLastOccur = datetime.GetTime(NULL);	/* done once per action on startup only */
 	pThis->iActionNbr = iActionNbr;
 	pthread_mutex_init(&pThis->mutAction, NULL);
@@ -350,13 +370,11 @@ finalize_it:
 /* action construction finalizer
  */
 rsRetVal
-actionConstructFinalize(action_t *pThis, struct nvlst *lst)
+actionConstructFinalize(action_t *__restrict__ const pThis, struct nvlst *lst)
 {
 	DEFiRet;
 	uchar pszAName[64]; /* friendly name of our action */
 
-	ASSERT(pThis != NULL);
-
 	if(!strcmp((char*)modGetName(pThis->pMod), "builtin:omdiscard")) {
 		/* discard actions will be optimized out */
 		FINALIZE;
@@ -364,14 +382,25 @@ actionConstructFinalize(action_t *pThis, struct nvlst *lst)
 	/* generate a friendly name for us action stats */
 	if(pThis->pszName == NULL) {
 		snprintf((char*) pszAName, sizeof(pszAName)/sizeof(uchar), "action %d", iActionNbr);
-	} else {
-		ustrncpy(pszAName, pThis->pszName, sizeof(pszAName));
-		pszAName[sizeof(pszAName)-1] = '\0'; /* to be on the save side */
+		pThis->pszName = ustrdup(pszAName);
 	}
 
+	/* cache transactional attribute */
+	pThis->isTransactional = pThis->pMod->mod.om.supportsTX;
+	if(pThis->isTransactional && pThis->eParamPassing != ACT_STRING_PASSING) {
+		errmsg.LogError(0, RS_RET_INVLD_OMOD, "action '%s'(%d) is transactional but "
+		                "uses invalid paramter passing mode -- disabling "
+				"action. This is probably caused by a pre-v7 "
+				"output module that needs upgrade.",
+				pThis->pszName, pThis->iActionNbr);
+		actionDisable(pThis);
+		ABORT_FINALIZE(RS_RET_INVLD_OMOD);
+	}
+
+
 	/* support statistics gathering */
 	CHKiRet(statsobj.Construct(&pThis->statsobj));
-	CHKiRet(statsobj.SetName(pThis->statsobj, pszAName));
+	CHKiRet(statsobj.SetName(pThis->statsobj, pThis->pszName));
 
 	STATSCOUNTER_INIT(pThis->ctrProcessed, pThis->mutCtrProcessed);
 	CHKiRet(statsobj.AddCounter(pThis->statsobj, UCHAR_CONSTANT("processed"),
@@ -381,18 +410,25 @@ actionConstructFinalize(action_t *pThis, struct nvlst *lst)
 	CHKiRet(statsobj.AddCounter(pThis->statsobj, UCHAR_CONSTANT("failed"),
 		ctrType_IntCtr, CTR_FLAG_RESETTABLE, &pThis->ctrFail));
 
+	STATSCOUNTER_INIT(pThis->ctrSuspend, pThis->mutCtrSuspend);
+	CHKiRet(statsobj.AddCounter(pThis->statsobj, UCHAR_CONSTANT("suspended"),
+		ctrType_IntCtr, CTR_FLAG_RESETTABLE, &pThis->ctrSuspend));
+	STATSCOUNTER_INIT(pThis->ctrSuspendDuration, pThis->mutCtrSuspendDuration);
+	CHKiRet(statsobj.AddCounter(pThis->statsobj, UCHAR_CONSTANT("suspended.duration"),
+		ctrType_IntCtr, 0, &pThis->ctrSuspendDuration));
+
+	STATSCOUNTER_INIT(pThis->ctrResume, pThis->mutCtrResume);
+	CHKiRet(statsobj.AddCounter(pThis->statsobj, UCHAR_CONSTANT("resumed"),
+		ctrType_IntCtr, CTR_FLAG_RESETTABLE, &pThis->ctrResume));
+
 	CHKiRet(statsobj.ConstructFinalize(pThis->statsobj));
 
 	/* create our queue */
 
 	/* generate a friendly name for the queue */
-	if(pThis->pszName == NULL) {
-		snprintf((char*) pszAName, sizeof(pszAName)/sizeof(uchar), "action %d queue",
-			 iActionNbr);
-	} else {
-		ustrncpy(pszAName, pThis->pszName, sizeof(pszAName));
-		pszAName[63] = '\0'; /* to be on the save side */
-	}
+	snprintf((char*) pszAName, sizeof(pszAName)/sizeof(uchar), "%s queue",
+		 pThis->pszName);
+
 	/* now check if we can run the action in "firehose mode" during stage one of 
 	 * its processing (that is before messages are enqueued into the action q).
 	 * This is only possible if some features, which require strict sequence, are
@@ -498,7 +534,7 @@ rsRetVal actionSetGlobalResumeInterval(int iNewVal)
  * returned string must not be modified.
  * rgerhards, 2009-05-07
  */
-static uchar *getActStateName(action_t *pThis, wti_t *pWti)
+static uchar *getActStateName(action_t * const pThis, wti_t * const pWti)
 {
 	switch(getActionState(pWti, pThis)) {
 		case ACT_STATE_RDY:
@@ -520,7 +556,7 @@ static uchar *getActStateName(action_t *pThis, wti_t *pWti)
 /* returns a suitable return code based on action state
  * rgerhards, 2009-05-07
  */
-static rsRetVal getReturnCode(action_t *pThis, wti_t *pWti)
+static rsRetVal getReturnCode(action_t * const pThis, wti_t * const pWti)
 {
 	DEFiRet;
 
@@ -558,7 +594,7 @@ static rsRetVal getReturnCode(action_t *pThis, wti_t *pWti)
  * rgerhards, 2007-08-02
  */
 static inline void
-actionSetState(action_t *pThis, wti_t *pWti, uint8_t newState)
+actionSetState(action_t * const pThis, wti_t * const pWti, uint8_t newState)
 {
 	setActionState(pWti, pThis, newState);
 	DBGPRINTF("Action %d transitioned to state: %s\n",
@@ -569,7 +605,7 @@ actionSetState(action_t *pThis, wti_t *pWti, uint8_t newState)
  * mostly a dummy...
  * rgerhards, 2007-08-02
  */
-static void actionCommitted(action_t *pThis, wti_t *pWti)
+static void actionCommitted(action_t * const pThis, wti_t * const pWti)
 {
 	actionSetState(pThis, pWti, ACT_STATE_RDY);
 }
@@ -578,25 +614,13 @@ static void actionCommitted(action_t *pThis, wti_t *pWti)
 /* set action to "rtry" state.
  * rgerhards, 2007-08-02
  */
-static void actionRetry(action_t *pThis, wti_t *pWti)
+static void actionRetry(action_t * const pThis, wti_t * const pWti)
 {
 	actionSetState(pThis, pWti, ACT_STATE_RTRY);
 	incActionResumeInRow(pWti, pThis);
 }
 
-
-/* Disable action, this means it will never again be usable
- * until rsyslog is reloaded. Use only as a last resort, but
- * depends on output module.
- * rgerhards, 2007-08-02
- */
-static void actionDisable(action_t *pThis)
-{
-	pThis->bDisabled = 1;
-}
-
-
-/* Suspend action, this involves changing the acton state as well
+/* Suspend action, this involves changing the action state as well
  * as setting the next retry time.
  * if we have more than 10 retries, we prolong the
  * retry interval. If something is really stalled, it will
@@ -604,18 +628,50 @@ static void actionDisable(action_t *pThis)
  * CPU time. TODO: maybe a config option for that?
  * rgerhards, 2007-08-02
  */
-static inline void actionSuspend(action_t *pThis, wti_t *pWti)
+static inline void
+actionSuspend(action_t * const pThis, wti_t * const pWti)
 {
 	time_t ttNow;
+	int suspendDuration;
+	char timebuf[32];
+
+	/* we need to defer setting the action's own bReportSuspension state until
+	 * after the full config has been processed. So the most simple case to do
+	 * that is here. It's not a performance problem, as it happens infrequently.
+	 * it's not a threading race problem, as always the same value will be written.
+	 */
+	if(pThis->bReportSuspension == -1)
+		pThis->bReportSuspension = bActionReportSuspension;
+	if(pThis->bReportSuspensionCont == -1) {
+		pThis->bReportSuspensionCont = bActionReportSuspensionCont;
+		if(pThis->bReportSuspensionCont == -1)
+			pThis->bReportSuspension = 1;
+	}
 
 	/* note: we can NOT use a cached timestamp, as time may have evolved
 	 * since caching, and this would break logic (and it actually did so!)
 	 */
 	datetime.GetTime(&ttNow);
-	pThis->ttResumeRtry = ttNow + pThis->iResumeInterval *
-		(getActionNbrResRtry(pWti, pThis) / 10 + 1);
+	suspendDuration = pThis->iResumeInterval * (getActionNbrResRtry(pWti, pThis) / 10 + 1);
+	pThis->ttResumeRtry = ttNow + suspendDuration;
 	actionSetState(pThis, pWti, ACT_STATE_SUSP);
-	DBGPRINTF("action suspended, earliest retry=%d\n", (int) pThis->ttResumeRtry);
+	pThis->ctrSuspendDuration += suspendDuration;
+	if(getActionNbrResRtry(pWti, pThis) == 0) {
+		STATSCOUNTER_INC(pThis->ctrSuspend, pThis->mutCtrSuspend);
+	}
+
+	if(   pThis->bReportSuspensionCont
+	   || (pThis->bReportSuspension && getActionNbrResRtry(pWti, pThis) == 0) ) {
+		ctime_r(&pThis->ttResumeRtry, timebuf);
+		timebuf[strlen(timebuf)-1] = '\0'; /* strip LF */
+		errmsg.LogMsg(0, RS_RET_SUSPENDED, LOG_WARNING,
+			      "action '%s' suspended, next retry is %s",
+			      pThis->pszName, timebuf);
+	}
+	DBGPRINTF("action '%s' suspended, earliest retry=%lld (now %lld), iNbrResRtry %d, "
+		  "duration %d\n",
+		  pThis->pszName, (long long) pThis->ttResumeRtry, (long long) ttNow,
+		  getActionNbrResRtry(pWti, pThis), suspendDuration);
 }
 
 
@@ -635,7 +691,7 @@ static inline void actionSuspend(action_t *pThis, wti_t *pWti)
  * of its inability to recover. -- rgerhards, 2010-04-26.
  */
 static rsRetVal
-actionDoRetry(action_t *pThis, wti_t *pWti)
+actionDoRetry(action_t * const pThis, wti_t * const pWti)
 {
 	int iRetries;
 	int iSleepPeriod;
@@ -646,9 +702,9 @@ actionDoRetry(action_t *pThis, wti_t *pWti)
 
 	iRetries = 0;
 	while((*pWti->pbShutdownImmediate == 0) && getActionState(pWti, pThis) == ACT_STATE_RTRY) {
-		DBGPRINTF("actionDoRetry: enter loop, iRetries=%d\n", iRetries);
+		DBGPRINTF("actionDoRetry: %s enter loop, iRetries=%d\n", pThis->pszName, iRetries);
 		iRet = pThis->pMod->tryResume(pWti->actWrkrInfo[pThis->iActionNbr].actWrkrData);
-		DBGPRINTF("actionDoRetry: action->tryResume returned %d\n", iRet);
+		DBGPRINTF("actionDoRetry: %s action->tryResume returned %d\n", pThis->pszName, iRet);
 		if((getActionResumeInRow(pWti, pThis) > 9) && (getActionResumeInRow(pWti, pThis) % 10 == 0)) {
 			bTreatOKasSusp = 1;
 			setActionResumeInRow(pWti, pThis, 0);
@@ -656,16 +712,25 @@ actionDoRetry(action_t *pThis, wti_t *pWti)
 			bTreatOKasSusp = 0;
 		}
 		if((iRet == RS_RET_OK) && (!bTreatOKasSusp)) {
-			DBGPRINTF("actionDoRetry: had success RDY again (iRet=%d)\n", iRet);
+			DBGPRINTF("actionDoRetry: %s had success RDY again (iRet=%d)\n",
+				  pThis->pszName, iRet);
+			if(pThis->bReportSuspension) {
+				errmsg.LogMsg(0, RS_RET_OK, LOG_INFO, "action '%s' "
+					      "resumed (module '%s')",
+					      pThis->pszName, pThis->pMod->pszName);
+			}
+			setActionJustResumed(pWti, pThis, 1);
 			actionSetState(pThis, pWti, ACT_STATE_RDY);
 		} else if(iRet == RS_RET_SUSPENDED || bTreatOKasSusp) {
 			/* max retries reached? */
-			DBGPRINTF("actionDoRetry: check for max retries, iResumeRetryCount %d, iRetries %d\n",
-				  pThis->iResumeRetryCount, iRetries);
+			DBGPRINTF("actionDoRetry: %s check for max retries, iResumeRetryCount "
+				  "%d, iRetries %d\n",
+				  pThis->pszName, pThis->iResumeRetryCount, iRetries);
 			if((pThis->iResumeRetryCount != -1 && iRetries >= pThis->iResumeRetryCount)) {
 				actionSuspend(pThis, pWti);
+				if(getActionNbrResRtry(pWti, pThis) < 20)
+					incActionNbrResRtry(pWti, pThis);
 			} else {
-				incActionNbrResRtry(pWti, pThis);
 				++iRetries;
 				iSleepPeriod = pThis->iResumeInterval;
 				srSleep(iSleepPeriod, 0);
@@ -688,13 +753,12 @@ finalize_it:
 
 
 static rsRetVal
-actionCheckAndCreateWrkrInstance(action_t *pThis, wti_t *pWti)
+actionCheckAndCreateWrkrInstance(action_t * const pThis, wti_t * const pWti)
 {
 	DEFiRet;
 	if(pWti->actWrkrInfo[pThis->iActionNbr].actWrkrData == NULL) {
-dbgprintf("DDDD: wti %p create new worker instance for action %d\n", pWti, pThis->iActionNbr);
-		DBGPRINTF("we need to create a new action worker instance for "
-			  "action %d\n", pThis->iActionNbr);
+		DBGPRINTF("wti %p: we need to create a new action worker instance for "
+			  "action %d\n", pWti, pThis->iActionNbr);
 		CHKiRet(pThis->pMod->mod.om.createWrkrInstance(&(pWti->actWrkrInfo[pThis->iActionNbr].actWrkrData),
 						               pThis->pModData));
 		pWti->actWrkrInfo[pThis->iActionNbr].pAction = pThis;
@@ -708,7 +772,7 @@ finalize_it:
  * changed to new action state engine -- rgerhards, 2009-05-07
  */
 static rsRetVal
-actionTryResume(action_t *pThis, wti_t *pWti)
+actionTryResume(action_t * const pThis, wti_t * const pWti)
 {
 	DEFiRet;
 	time_t ttNow = NO_TIME_PROVIDED;
@@ -748,11 +812,10 @@ finalize_it:
  * rgerhards, 2009-05-07
  */
 static inline rsRetVal
-actionPrepare(action_t *pThis, wti_t *pWti)
+actionPrepare(action_t *__restrict__ const pThis, wti_t *__restrict__ const pWti)
 {
 	DEFiRet;
 
-	assert(pThis != NULL);
 	CHKiRet(actionCheckAndCreateWrkrInstance(pThis, pWti));
 	CHKiRet(actionTryResume(pThis, pWti));
 
@@ -760,7 +823,6 @@ actionPrepare(action_t *pThis, wti_t *pWti)
 	 * action state accordingly
 	 */
 	if(getActionState(pWti, pThis) == ACT_STATE_RDY) {
-dbgprintf("DDDDD: calling beginTransaction for action %d\n", pThis->iActionNbr);
 		iRet = pThis->pMod->mod.om.beginTransaction(pWti->actWrkrInfo[pThis->iActionNbr].actWrkrData);
 		switch(iRet) {
 			case RS_RET_OK:
@@ -823,43 +885,50 @@ static rsRetVal actionDbgPrint(action_t *pThis)
  * rgerhards, 2009-05-07
  */
 static rsRetVal
-prepareDoActionParams(action_t *pAction, wti_t *pWti, msg_t *pMsg, struct syslogTime *ttNow)
+prepareDoActionParams(action_t * __restrict__ const pAction,
+		      wti_t * __restrict__ const pWti,
+		      msg_t *__restrict__ const pMsg,
+		      struct syslogTime *ttNow)
 {
 	int i;
 	struct json_object *json;
 	actWrkrIParams_t *iparams;
-	actWrkrInfo_t *pWrkrInfo;
+	actWrkrInfo_t *__restrict__ pWrkrInfo;
 	DEFiRet;
 
 	pWrkrInfo = &(pWti->actWrkrInfo[pAction->iActionNbr]);
-	if(pAction->eParamPassing == ACT_STRING_PASSING) {
+	if(pAction->isTransactional) {
 		CHKiRet(wtiNewIParam(pWti, pAction, &iparams));
-	}
-
-	/* here we must loop to process all requested strings */
-	for(i = 0 ; i < pAction->iNumTpls ; ++i) {
-dbgprintf("DDDDD: generating template #%d\n", i);
-		switch(pAction->eParamPassing) {
+		for(i = 0 ; i < pAction->iNumTpls ; ++i) {
+			CHKiRet(tplToString(pAction->ppTpl[i], pMsg, 
+					    &actParam(iparams, pAction->iNumTpls, 0, i),
+				            ttNow));
+		}
+	} else {
+		for(i = 0 ; i < pAction->iNumTpls ; ++i) {
+			switch(pAction->eParamPassing) {
 			case ACT_STRING_PASSING:
-				iparams->msgFlags = pMsg->msgFlags;
-				CHKiRet(tplToString(pAction->ppTpl[i], pMsg, &(iparams->staticActStrings[i]),
-					&iparams->staticLenStrings[i], ttNow));
-				iparams->staticActParams[i] = iparams->staticActStrings[i];
+				CHKiRet(tplToString(pAction->ppTpl[i], pMsg,
+					   &(pWrkrInfo->p.nontx.actParams[i]),
+					   ttNow));
 				break;
 			case ACT_ARRAY_PASSING:
-				CHKiRet(tplToArray(pAction->ppTpl[i], pMsg, (uchar***) &(pWrkrInfo->staticActParams[i]), ttNow));
+				CHKiRet(tplToArray(pAction->ppTpl[i], pMsg,
+					(uchar***) &(pWrkrInfo->p.nontx.actParams[i].param), ttNow));
 				break;
 			case ACT_MSG_PASSING:
-				pWrkrInfo->staticActParams[i] = (void*) pMsg;
+				pWrkrInfo->p.nontx.actParams[i].param = (void*) pMsg;
 				break;
 			case ACT_JSON_PASSING:
 				CHKiRet(tplToJSON(pAction->ppTpl[i], pMsg, &json, ttNow));
-				pWrkrInfo->staticActParams[i] = (void*) json;
+				pWrkrInfo->p.nontx.actParams[i].param = (void*) json;
 				break;
-			default:dbgprintf("software bug/error: unknown pAction->eParamPassing %d in prepareDoActionParams\n",
+			default:dbgprintf("software bug/error: unknown pAction->eParamPassing "
+					  "%d in prepareDoActionParams\n",
 					   (int) pAction->eParamPassing);
 				assert(0); /* software bug if this happens! */
 				break;
+			}
 		}
 	}
 
@@ -869,11 +938,11 @@ finalize_it:
 
 
 static void
-releaseDoActionParams(action_t *pAction, wti_t *pWti)
+releaseDoActionParams(action_t *__restrict__ const pAction, wti_t *__restrict__ const pWti)
 {
 	int jArr;
 	int j;
-	actWrkrInfo_t *pWrkrInfo;
+	actWrkrInfo_t *__restrict__ pWrkrInfo;
 	uchar ***ppMsgs;
 
 	if(pAction->eParamPassing == ACT_STRING_PASSING || pAction->eParamPassing == ACT_MSG_PASSING)
@@ -882,7 +951,7 @@ releaseDoActionParams(action_t *pAction, wti_t *pWti)
 	pWrkrInfo = &(pWti->actWrkrInfo[pAction->iActionNbr]);
 	switch(pAction->eParamPassing) {
 	case ACT_ARRAY_PASSING:
-		ppMsgs = (uchar***) pWrkrInfo->staticActParams;
+		ppMsgs = (uchar***) pWrkrInfo->p.nontx.actParams[0].param;
 		for(j = 0 ; j < pAction->iNumTpls ; ++j) {
 			if(((uchar**)ppMsgs)[j] != NULL) {
 				jArr = 0;
@@ -899,11 +968,12 @@ releaseDoActionParams(action_t *pAction, wti_t *pWti)
 	case ACT_JSON_PASSING:
 		for(j = 0 ; j < pAction->iNumTpls ; ++j) {
 			json_object_put((struct json_object*)
-					pWrkrInfo->staticActParams[j]);
-			pWrkrInfo->staticActParams[j] = NULL;
+					pWrkrInfo->p.nontx.actParams[j].param);
+			pWrkrInfo->p.nontx.actParams[j].param = NULL;
 		}
 		break;
 	case ACT_STRING_PASSING:
+		/* strings are destructed when the worker terminates */
 	case ACT_MSG_PASSING:
 		/* can never happen, just to keep compiler happy! */
 		break;
@@ -912,38 +982,45 @@ releaseDoActionParams(action_t *pAction, wti_t *pWti)
 done:	return;
 }
 
-
-/* call the DoAction output plugin entry point
- * rgerhards, 2008-01-28
+/* This is used in resume processing. We only finally know that a resume
+ * worked when we have been able to actually process a messages. As such,
+ * we need to do some cleanup and status tracking in that case.
  */
+static void
+actionSetActionWorked(action_t *__restrict__ const pThis, wti_t *__restrict__ const pWti)
+{
+	setActionResumeInRow(pWti, pThis, 0);
+
+	if(getActionJustResumed(pWti, pThis)) {
+		/* OK, we *really* could resume, so tell user! */
+		if(pThis->bReportSuspension) {
+			errmsg.LogMsg(0, RS_RET_RESUMED, LOG_INFO, "action '%s' "
+				      "resumed (module '%s')",
+				      pThis->pszName, pThis->pMod->pszName);
+		}
+		setActionJustResumed(pWti, pThis, 0);
+	}
+}
+
 static rsRetVal
-actionCallDoAction(action_t *pThis, int msgFlags, void *actParams, wti_t *pWti)
+handleActionExecResult(action_t *__restrict__ const pThis,
+	wti_t *__restrict__ const pWti,
+	const rsRetVal ret)
 {
 	DEFiRet;
-
-	ASSERT(pThis != NULL);
-
-	DBGPRINTF("entering actionCalldoAction(), state: %s, actionNbr %d\n",
-		  getActStateName(pThis, pWti), pThis->iActionNbr);
-
-	CHKiRet(actionCheckAndCreateWrkrInstance(pThis, pWti));
-
-	pThis->bHadAutoCommit = 0;
-	iRet = pThis->pMod->mod.om.doAction(actParams, msgFlags,
-				            pWti->actWrkrInfo[pThis->iActionNbr].actWrkrData);
-	switch(iRet) {
+	switch(ret) {
 		case RS_RET_OK:
 			actionCommitted(pThis, pWti);
-			setActionResumeInRow(pWti, pThis, 0);
+			actionSetActionWorked(pThis, pWti); /* we had a successful call! */
 			break;
 		case RS_RET_DEFER_COMMIT:
-			setActionResumeInRow(pWti, pThis, 0);
+			actionSetActionWorked(pThis, pWti); /* we had a successful call! */
 			/* we are done, action state remains the same */
 			break;
 		case RS_RET_PREVIOUS_COMMITTED:
 			/* action state remains the same, but we had a commit. */
 			pThis->bHadAutoCommit = 1;
-			setActionResumeInRow(pWti, pThis, 0);
+			actionSetActionWorked(pThis, pWti); /* we had a successful call! */
 			break;
 		case RS_RET_SUSPENDED:
 			actionRetry(pThis, pWti);
@@ -954,6 +1031,7 @@ actionCallDoAction(action_t *pThis, int msgFlags, void *actParams, wti_t *pWti)
 		default:/* permanent failure of this message - no sense in retrying. This is
 			 * not yet handled (but easy TODO)
 			 */
+			iRet = ret;
 			FINALIZE;
 	}
 	iRet = getReturnCode(pThis, pWti);
@@ -962,13 +1040,65 @@ finalize_it:
 	RETiRet;
 }
 
+/* call the DoAction output plugin entry point
+ * rgerhards, 2008-01-28
+ */
+static rsRetVal
+actionCallDoAction(action_t *__restrict__ const pThis,
+	actWrkrIParams_t *__restrict__ const iparams,
+	wti_t *__restrict__ const pWti)
+{
+	uchar *param[CONF_OMOD_NUMSTRINGS_MAXSIZE];
+	int i;
+	DEFiRet;
+
+	DBGPRINTF("entering actionCalldoAction(), state: %s, actionNbr %d\n",
+		  getActStateName(pThis, pWti), pThis->iActionNbr);
+
+	pThis->bHadAutoCommit = 0;
+	/* for this interface, we need to emulate the old style way
+	 * of parameter passing.
+	 */
+	for(i = 0 ; i < pThis->iNumTpls ; ++i) {
+		param[i] = actParam(iparams, pThis->iNumTpls, 0, i).param;
+	}
+
+	iRet = pThis->pMod->mod.om.doAction(param,
+				            pWti->actWrkrInfo[pThis->iActionNbr].actWrkrData);
+	iRet = handleActionExecResult(pThis, pWti, iRet);
+	RETiRet;
+}
+
+
+/* call the commitTransaction output plugin entry point */
+static rsRetVal
+actionCallCommitTransaction(action_t * const pThis,
+	const actWrkrInfo_t *const wrkrInfo,
+	wti_t *const pWti)
+{
+	DEFiRet;
+
+	ASSERT(pThis != NULL);
+
+	DBGPRINTF("entering actionCallCommitTransaction(), state: %s, actionNbr %d, "
+		  "nMsgs %u\n",
+		  getActStateName(pThis, pWti), pThis->iActionNbr,
+		  wrkrInfo->p.tx.currIParam);
+
+	iRet = pThis->pMod->mod.om.commitTransaction(
+		    pWti->actWrkrInfo[pThis->iActionNbr].actWrkrData,
+		    wrkrInfo->p.tx.iparams, wrkrInfo->p.tx.currIParam);
+	iRet = handleActionExecResult(pThis, pWti, iRet);
+	RETiRet;
+}
+
 
 /* process a message
  * this readies the action and then calls doAction()
  * rgerhards, 2008-01-28
  */
 rsRetVal
-actionProcessMessage(action_t *pThis, int msgFlags, void *actParams, wti_t *pWti)
+actionProcessMessage(action_t * const pThis, void *actParams, wti_t * const pWti)
 {
 	DEFiRet;
 
@@ -976,7 +1106,7 @@ actionProcessMessage(action_t *pThis, int msgFlags, void *actParams, wti_t *pWti
 	if(pThis->pMod->mod.om.SetShutdownImmdtPtr != NULL)
 		pThis->pMod->mod.om.SetShutdownImmdtPtr(pThis->pModData, pWti->pbShutdownImmediate);
 	if(getActionState(pWti, pThis) == ACT_STATE_ITX)
-		CHKiRet(actionCallDoAction(pThis, msgFlags, actParams, pWti));
+		CHKiRet(actionCallDoAction(pThis, actParams, pWti));
 
 	iRet = getReturnCode(pThis, pWti);
 finalize_it:
@@ -986,64 +1116,42 @@ finalize_it:
 
 /* the following functions simulates a potential future new omo callback */
 static rsRetVal
-doTransaction(action_t *pThis, wti_t *pWti)
+doTransaction(action_t *__restrict__ const pThis, wti_t *__restrict__ const pWti)
 {
 	actWrkrInfo_t *wrkrInfo;
-	actWrkrIParams_t *iparamCurr;
 	int i;
 	DEFiRet;
 
 	wrkrInfo = &(pWti->actWrkrInfo[pThis->iActionNbr]);
-	dbgprintf("DDDD: doTransaction: action %d, currIParams %d\n", pThis->iActionNbr, wrkrInfo->currIParam);
-	for(i = 0 ; i < wrkrInfo->currIParam ; ++i) {
-		iparamCurr = wrkrInfo->iparams + i;
-		iRet = actionProcessMessage(pThis, iparamCurr->msgFlags,
-					    iparamCurr->staticActParams,
-					    pWti);
-		dbgprintf("DDDD: doTransaction loop, iRet %d\n", iRet);
-	}
-	RETiRet;
-}
-
-
-static void
-actionFreeParams(action_t *pThis, wti_t *pWti)
-{
-	actWrkrInfo_t *wrkrInfo;
-	actWrkrIParams_t *iparamCurr;
-	int i;
-	int j;
-
-	wrkrInfo = &(pWti->actWrkrInfo[pThis->iActionNbr]);
-	dbgprintf("DDDD: actionFreeParams: action %d, currIParam %d\n", pThis->iActionNbr, wrkrInfo->currIParam);
-	for(i = 0 ; i < wrkrInfo->currIParam ; ++i) {
-		iparamCurr = wrkrInfo->iparams + i;
-		for(j = 0 ; j < CONF_OMOD_NUMSTRINGS_MAXSIZE ; ++j) {
-			/* TODO: we can save time by not freeing everything,
-			 * but that's left for a later optimization.
+	if(pThis->pMod->mod.om.commitTransaction != NULL) {
+		DBGPRINTF("doTransaction: have commitTransaction IF, using that, pWrkrInfo %p\n", wrkrInfo);
+		CHKiRet(actionCallCommitTransaction(pThis, wrkrInfo, pWti));
+	} else { /* note: this branch is for compatibility with old TX modules */
+		DBGPRINTF("doTransaction: action %d, currIParam %d\n",
+			   pThis->iActionNbr, wrkrInfo->p.tx.currIParam);
+		for(i = 0 ; i < wrkrInfo->p.tx.currIParam ; ++i) {
+			/* Note: we provide the message's base iparam - actionProcessMessage()
+			 * uses this as *base* address.
 			 */
-			free(iparamCurr->staticActStrings[j]);
-			iparamCurr->staticActStrings[j] = NULL;
-			iparamCurr->staticLenStrings[j] = 0;
+			iRet = actionProcessMessage(pThis,
+				&actParam(wrkrInfo->p.tx.iparams, pThis->iNumTpls, i, 0), pWti);
 		}
 	}
-	wrkrInfo->currIParam = 0; /* reset to beginning */
-	releaseDoActionParams(pThis, pWti);
+finalize_it:
+	RETiRet;
 }
 
 
 /* Commit try committing (do not handle retry processing and such) */
 static rsRetVal
-actionTryCommit(action_t *pThis, wti_t *pWti)
+actionTryCommit(action_t *__restrict__ const pThis, wti_t *__restrict__ const pWti)
 {
-	//actWrkrInfo_t *wrkrInfo;
 	DEFiRet;
 
 	doTransaction(pThis, pWti);
 
 	CHKiRet(actionPrepare(pThis, pWti));
 	if(getActionState(pWti, pThis) == ACT_STATE_ITX) {
-dbgprintf("DDDDD: calling endTransaction for action %d\n", pThis->iActionNbr);
 		iRet = pThis->pMod->mod.om.endTransaction(pWti->actWrkrInfo[pThis->iActionNbr].actWrkrData);
 		switch(iRet) {
 			case RS_RET_OK:
@@ -1074,7 +1182,7 @@ dbgprintf("DDDDD: calling endTransaction for action %d\n", pThis->iActionNbr);
 	iRet = getReturnCode(pThis, pWti);
 
 finalize_it:
-	actionFreeParams(pThis, pWti);
+	pWti->actWrkrInfo[pThis->iActionNbr].p.tx.currIParam = 0; /* reset to beginning */
 	RETiRet;
 }
 
@@ -1084,11 +1192,18 @@ finalize_it:
  * rgerhards, 2013-11-06
  */
 static rsRetVal
-actionCommit(action_t *pThis, wti_t *pWti)
+actionCommit(action_t *__restrict__ const pThis, wti_t *__restrict__ const pWti)
 {
 	sbool bDone;
 	DEFiRet;
 
+	if(!pThis->isTransactional ||
+	   pWti->actWrkrInfo[pThis->iActionNbr].p.tx.currIParam == 0 ||
+	   getActionState(pWti, pThis) == ACT_STATE_SUSP
+	   ) {
+		FINALIZE;
+	}
+
 	/* even more TODO:
 		This is the place where retry processing needs to go in. If the action
 		permanently fails, we should - as a new feature - add the capability to
@@ -1108,7 +1223,13 @@ actionCommit(action_t *pThis, wti_t *pWti)
 		DBGPRINTF("actionCommit, in retry loop, iRet %d\n", iRet);
 		if(iRet == RS_RET_FORCE_TERM) {
 			ABORT_FINALIZE(RS_RET_FORCE_TERM);
-		} else if(iRet == RS_RET_OK || iRet == RS_RET_ACTION_FAILED) {
+		} else if(iRet == RS_RET_OK ||
+			  iRet == RS_RET_SUSPENDED ||
+			  iRet == RS_RET_ACTION_FAILED) {
+			bDone = 1;
+		}
+		if(getActionState(pWti, pThis) == ACT_STATE_RDY ||
+		   getActionState(pWti, pThis) == ACT_STATE_SUSP) {
 			bDone = 1;
 		}
 	} while(!bDone);
@@ -1118,16 +1239,20 @@ finalize_it:
 
 /* Commit all active transactions in *DIRECT mode* */
 void
-actionCommitAllDirect(wti_t *pWti)
+actionCommitAllDirect(wti_t *__restrict__ const pWti)
 {
 	int i;
 	action_t *pAction;
 
 	for(i = 0 ; i < iActionNbr ; ++i) {
-		dbgprintf("DDDD: actionCommitAll: action %d, state %u, nbr to commit %d\n",
-			  i, getActionStateByNbr(pWti, i), pWti->actWrkrInfo->currIParam);
 		pAction = pWti->actWrkrInfo[i].pAction;
-		if(pAction != NULL && pAction->pQueue->qType == QUEUETYPE_DIRECT)
+		if(pAction == NULL)
+			continue;
+		DBGPRINTF("actionCommitAll: action %d, state %u, nbr to commit %d "
+			  "isTransactional %d\n",
+			  i, getActionStateByNbr(pWti, i), pWti->actWrkrInfo->p.tx.currIParam,
+			  pAction->isTransactional);
+		if(pAction->pQueue->qType == QUEUETYPE_DIRECT)
 			actionCommit(pAction, pWti);
 	}
 }
@@ -1137,7 +1262,10 @@ actionCommitAllDirect(wti_t *pWti)
  * queue thread if the action queue is set to "direct".
  */
 static rsRetVal
-processMsgMain(action_t *pAction, wti_t *pWti, msg_t *pMsg, struct syslogTime *ttNow)
+processMsgMain(action_t *__restrict__ const pAction,
+	wti_t *__restrict__ const pWti,
+	msg_t *__restrict__ const pMsg,
+	struct syslogTime *ttNow)
 {
 	DEFiRet;
 
@@ -1147,35 +1275,37 @@ processMsgMain(action_t *pAction, wti_t *pWti, msg_t *pMsg, struct syslogTime *t
 		FINALIZE;
 	}
 
-dbgprintf("DDDD: processMsgMain[act %d], %s\n", pAction->iActionNbr, pMsg->pszRawMsg);
 	iRet = prepareDoActionParams(pAction, pWti, pMsg, ttNow);
-	if(pAction->eParamPassing == ACT_STRING_PASSING) {
+
+	if(pAction->isTransactional) {
 		pWti->actWrkrInfo[pAction->iActionNbr].pAction = pAction;
-		dbgprintf("DDDD: action %d is string passing - executing in commit phase\n", pAction->iActionNbr);
+		DBGPRINTF("action %d is transactional - executing in commit phase\n", pAction->iActionNbr);
 		actionPrepare(pAction, pWti);
 		iRet = getReturnCode(pAction, pWti);
 		FINALIZE;
 	}
 
-	iRet = actionProcessMessage(pAction, pMsg->msgFlags,
-				    pWti->actWrkrInfo[pAction->iActionNbr].staticActParams,
+	iRet = actionProcessMessage(pAction,
+				    pWti->actWrkrInfo[pAction->iActionNbr].p.nontx.actParams,
 				    pWti);
 	releaseDoActionParams(pAction, pWti);
 finalize_it:
-	if(pWti->execState.bDoAutoCommit)
-		iRet = actionCommit(pAction, pWti);
+	if(iRet == RS_RET_OK) {
+		if(pWti->execState.bDoAutoCommit)
+			iRet = actionCommit(pAction, pWti);
+	}
 	pWti->execState.bPrevWasSuspended = (iRet == RS_RET_SUSPENDED || iRet == RS_RET_ACTION_FAILED);
-dbgprintf("DDDD: bPrevWasSuspended now %d, action state %d\n", (int)pWti->execState.bPrevWasSuspended, getActionState(pWti, pAction));
 	RETiRet;
 }
 
 /* This entry point is called by the ACTION queue (not main queue!)
  */
 static rsRetVal
-processBatchMain(void *pVoid, batch_t *pBatch, wti_t *pWti)
+processBatchMain(void *__restrict__ const pVoid,
+	batch_t *__restrict__ const pBatch,
+	wti_t *__restrict__ const pWti)
 {
-	action_t *pAction = (action_t*) pVoid;
-	msg_t *pMsg;
+	action_t *__restrict__ const pAction = (action_t*__restrict__ const) pVoid;
 	int i;
 	struct syslogTime ttNow;
 	DEFiRet;
@@ -1186,16 +1316,12 @@ processBatchMain(void *pVoid, batch_t *pBatch, wti_t *pWti)
 
 	for(i = 0 ; i < batchNumMsgs(pBatch) && !*pWti->pbShutdownImmediate ; ++i) {
 		if(batchIsValidElem(pBatch, i)) {
-			pMsg = pBatch->pElem[i].pMsg;
-			iRet = processMsgMain(pAction, pWti, pMsg, &ttNow);
-		// TODO: we must refactor this!  flag messages as committed
-		batchSetElemState(pBatch, i, BATCH_STATE_COMM);
+			iRet = processMsgMain(pAction, pWti, pBatch->pElem[i].pMsg, &ttNow);
+			batchSetElemState(pBatch, i, BATCH_STATE_COMM);
 		}
 	}
 
-	if(!pWti->execState.bDoAutoCommit)
-		iRet = actionCommit(pAction, pWti);
-dbgprintf("DDDD: processBatchMain - end\n");
+	iRet = actionCommit(pAction, pWti);
 	RETiRet;
 }
 
@@ -1205,7 +1331,7 @@ dbgprintf("DDDD: processBatchMain - end\n");
  * to any current doAction() processing.
  */
 rsRetVal
-actionCallHUPHdlr(action_t *pAction)
+actionCallHUPHdlr(action_t * const pAction)
 {
 	DEFiRet;
 
@@ -1258,7 +1384,7 @@ static rsRetVal setActionQueType(void __attribute__((unused)) *pVal, uchar *pszT
  * rgerhards, 2010-06-08
  */
 static rsRetVal
-doSubmitToActionQ(action_t *pAction, wti_t *pWti, msg_t *pMsg)
+doSubmitToActionQ(action_t * const pAction, wti_t * const pWti, msg_t *pMsg)
 {
 	struct syslogTime ttNow; // TODO: think if we can buffer this in pWti
 	DEFiRet;
@@ -1287,7 +1413,7 @@ doSubmitToActionQ(action_t *pAction, wti_t *pWti, msg_t *pMsg)
  * be filtered out before calling us (what is done currently!).
  */
 rsRetVal
-actionWriteToAction(action_t *pAction, msg_t *pMsg, wti_t *pWti)
+actionWriteToAction(action_t * const pAction, msg_t *pMsg, wti_t * const pWti)
 {
 	DEFiRet;
 
@@ -1354,7 +1480,7 @@ finalize_it:
  */
 #pragma GCC diagnostic ignored "-Wempty-body"
 static rsRetVal
-doSubmitToActionQComplex(action_t *pAction, wti_t *pWti, msg_t *pMsg)
+doSubmitToActionQComplex(action_t * const pAction, wti_t * const pWti, msg_t *pMsg)
 {
 	DEFiRet;
 
@@ -1389,7 +1515,7 @@ finalize_it:
 DEFFUNC_llExecFunc(doActivateActions)
 {
 	rsRetVal localRet;
-	action_t *pThis = (action_t*) pData;
+	action_t * const pThis = (action_t*) pData;
 	BEGINfunc
 	localRet = qqueueStart(pThis->pQueue);
 	if(localRet != RS_RET_OK) {
@@ -1430,7 +1556,7 @@ activateActions(void)
  * rgerhards, 2010-06-08
  */
 static rsRetVal
-doSubmitToActionQNotAllMark(action_t *pAction, wti_t *pWti, msg_t *pMsg)
+doSubmitToActionQNotAllMark(action_t * const pAction, wti_t * const pWti, msg_t * const pMsg)
 {
 	int doProcess = 1;
 	time_t lastAct;
@@ -1471,7 +1597,7 @@ doSubmitToActionQNotAllMark(action_t *pAction, wti_t *pWti, msg_t *pMsg)
  * rgerhards, 2011-08-01
  */
 static rsRetVal
-actionApplyCnfParam(action_t *pAction, struct cnfparamvals *pvals)
+actionApplyCnfParam(action_t * const pAction, struct cnfparamvals * const pvals)
 {
 	int i;
 	
@@ -1496,6 +1622,10 @@ actionApplyCnfParam(action_t *pAction, struct cnfparamvals *pvals)
 			pAction->bRepMsgHasMsg = pvals[i].val.d.n;
 		} else if(!strcmp(pblk.descr[i].name, "action.resumeretrycount")) {
 			pAction->iResumeRetryCount = pvals[i].val.d.n;
+		} else if(!strcmp(pblk.descr[i].name, "action.reportsuspension")) {
+			pAction->bReportSuspension = (int) pvals[i].val.d.n;
+		} else if(!strcmp(pblk.descr[i].name, "action.reportsuspensioncontinuation")) {
+			pAction->bReportSuspensionCont = (int) pvals[i].val.d.n;
 		} else if(!strcmp(pblk.descr[i].name, "action.resumeinterval")) {
 			pAction->iResumeInterval = pvals[i].val.d.n;
 		} else {
@@ -1515,7 +1645,7 @@ actionApplyCnfParam(action_t *pAction, struct cnfparamvals *pvals)
 rsRetVal
 addAction(action_t **ppAction, modInfo_t *pMod, void *pModData,
 	  omodStringRequest_t *pOMSR, struct cnfparamvals *actParams,
-	  struct nvlst *lst)
+	  struct nvlst * const lst)
 {
 	DEFiRet;
 	int i;
