@@ -4,7 +4,7 @@
  * File begun on 2007-08-06 by RGerhards (extracted from syslogd.c, which
  * was under BSD license at the time of rsyslog fork)
  *
- * Copyright 2007-2012 Adiscon GmbH.
+ * Copyright 2007-2013 Adiscon GmbH.
  *
  * This file is part of rsyslog.
  *
@@ -28,11 +28,10 @@
 #include "syslogd-types.h"
 #include "queue.h"
 
-/* external data - this is to be removed when we change the action
- * object interface (will happen some time..., at latest when the
- * config file format is changed). -- rgerhards, 2008-01-28
- */
+/* external data */
 extern int glbliActionResumeRetryCount;
+extern int bActionReportSuspension;
+extern int bActionReportSuspensionCont;
 
 
 /* the following struct defines the action object data structure
@@ -45,8 +44,11 @@ struct action_s {
 	int	iActionNbr;	/* this action's number (ID) */
 	sbool	bExecWhenPrevSusp;/* execute only when previous action is suspended? */
 	sbool	bWriteAllMarkMsgs;/* should all mark msgs be written (not matter how recent the action was executed)? */
+	sbool	bReportSuspension;/* should suspension (and reactivation) of the action reported */
+	sbool	bReportSuspensionCont;
 	sbool	bHadAutoCommit;	/* did an auto-commit happen during doAction()? */
 	sbool	bDisabled;
+	sbool	isTransactional;
 	int	iSecsExecOnceInterval; /* if non-zero, minimum seconds to wait until action is executed again */
 	time_t	ttResumeRtry;	/* when is it time to retry the resume? */
 	int	iResumeInterval;/* resume interval for this action */
@@ -74,6 +76,9 @@ struct action_s {
 	statsobj_t *statsobj;
 	STATSCOUNTER_DEF(ctrProcessed, mutCtrProcessed);
 	STATSCOUNTER_DEF(ctrFail, mutCtrFail);
+	STATSCOUNTER_DEF(ctrSuspend, mutCtrSuspend);
+	STATSCOUNTER_DEF(ctrSuspendDuration, mutCtrSuspendDuration);
+	STATSCOUNTER_DEF(ctrResume, mutCtrResume);
 };
 
 
