@@ -1,4 +1,4 @@
-/* A Bison parser, made by GNU Bison 2.6.1.  */
+/* A Bison parser, made by GNU Bison 2.6.4.  */
 
 /* Bison implementation for Yacc-like parsers in C
    
@@ -44,7 +44,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.6.1"
+#define YYBISON_VERSION "2.6.4"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -62,9 +62,10 @@
 
 
 /* Copy the first part of user declarations.  */
-/* Line 336 of yacc.c  */
+/* Line 358 of yacc.c  */
 #line 31 "grammar.y"
 
+#include "config.h"
 #include <stdio.h>
 #include <libestr.h>
 #include "rainerscript.h"
@@ -76,8 +77,8 @@ extern int yylineno;
 extern int yylex(void);
 extern int yyerror(char*);
 
-/* Line 336 of yacc.c  */
-#line 81 "grammar.c"
+/* Line 358 of yacc.c  */
+#line 82 "grammar.c"
 
 # ifndef YY_NULL
 #  if defined __cplusplus && 201103L <= __cplusplus
@@ -97,8 +98,8 @@ extern int yyerror(char*);
 
 /* In a future release of Bison, this section will be replaced
    by #include "y.tab.h".  */
-#ifndef YY_Y_TAB_H
-# define YY_Y_TAB_H
+#ifndef YY_YY_GRAMMAR_H_INCLUDED
+# define YY_YY_GRAMMAR_H_INCLUDED
 /* Enabling traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
@@ -202,8 +203,8 @@ extern int yydebug;
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
 {
-/* Line 350 of yacc.c  */
-#line 44 "grammar.y"
+/* Line 374 of yacc.c  */
+#line 45 "grammar.y"
 
 	char *s;
 	long long n;
@@ -219,8 +220,8 @@ typedef union YYSTYPE
 	struct cnffparamlst *fparams;
 
 
-/* Line 350 of yacc.c  */
-#line 224 "grammar.c"
+/* Line 374 of yacc.c  */
+#line 225 "grammar.c"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
@@ -243,12 +244,12 @@ int yyparse ();
 #endif
 #endif /* ! YYPARSE_PARAM */
 
-#endif /* !YY_Y_TAB_H  */
+#endif /* !YY_YY_GRAMMAR_H_INCLUDED  */
 
 /* Copy the second part of user declarations.  */
 
-/* Line 353 of yacc.c  */
-#line 252 "grammar.c"
+/* Line 377 of yacc.c  */
+#line 253 "grammar.c"
 
 #ifdef short
 # undef short
@@ -301,24 +302,24 @@ typedef short int yytype_int16;
 # if defined YYENABLE_NLS && YYENABLE_NLS
 #  if ENABLE_NLS
 #   include <libintl.h> /* INFRINGES ON USER NAME SPACE */
-#   define YY_(msgid) dgettext ("bison-runtime", msgid)
+#   define YY_(Msgid) dgettext ("bison-runtime", Msgid)
 #  endif
 # endif
 # ifndef YY_
-#  define YY_(msgid) msgid
+#  define YY_(Msgid) Msgid
 # endif
 #endif
 
 /* Suppress unused-variable warnings by "using" E.  */
 #if ! defined lint || defined __GNUC__
-# define YYUSE(e) ((void) (e))
+# define YYUSE(E) ((void) (E))
 #else
-# define YYUSE(e) /* empty */
+# define YYUSE(E) /* empty */
 #endif
 
 /* Identity function, used to suppress warnings about constant conditions.  */
 #ifndef lint
-# define YYID(n) (n)
+# define YYID(N) (N)
 #else
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
@@ -569,14 +570,14 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
-       0,   122,   122,   123,   124,   125,   126,   127,   128,   129,
-     130,   134,   138,   139,   140,   141,   142,   143,   144,   145,
-     146,   147,   148,   149,   150,   151,   155,   159,   160,   161,
-     162,   163,   164,   165,   166,   168,   169,   170,   171,   172,
-     173,   174,   175,   176,   177,   178,   179,   180,   181,   182,
-     183,   184,   185,   186,   187,   188,   189,   190,   191,   192,
-     193,   194,   195,   196,   197,   198,   199,   200,   201,   202,
-     203,   204
+       0,   123,   123,   124,   125,   126,   127,   128,   129,   130,
+     131,   135,   139,   140,   141,   142,   143,   144,   145,   146,
+     147,   148,   149,   150,   151,   152,   156,   160,   161,   162,
+     163,   164,   165,   166,   167,   169,   170,   171,   172,   173,
+     174,   175,   176,   177,   178,   179,   180,   181,   182,   183,
+     184,   185,   186,   187,   188,   189,   190,   191,   192,   193,
+     194,   195,   196,   197,   198,   199,   200,   201,   202,   203,
+     204,   205
 };
 #endif
 
@@ -735,10 +736,10 @@ static const yytype_uint8 yytable[] =
       74,    75,    76,    77,    78,    79
 };
 
-#define yypact_value_is_default(yystate) \
-  ((yystate) == (-50))
+#define yypact_value_is_default(Yystate) \
+  (!!((Yystate) == (-50)))
 
-#define yytable_value_is_error(yytable_value) \
+#define yytable_value_is_error(Yytable_value) \
   YYID (0)
 
 static const yytype_int8 yycheck[] =
@@ -1345,6 +1346,15 @@ yydestruct (yymsg, yytype, yyvaluep)
 /* The lookahead symbol.  */
 int yychar;
 
+
+#ifndef YYLVAL_INITIALIZE
+# define YYLVAL_INITIALIZE()
+#endif
+#ifndef YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
+# define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
+# define YY_IGNORE_MAYBE_UNINITIALIZED_END
+#endif
+
 /* The semantic value of the lookahead symbol.  */
 YYSTYPE yylval;
 
@@ -1404,7 +1414,7 @@ yyparse ()
   int yyn;
   int yyresult;
   /* Lookahead token as an internal (translated) token number.  */
-  int yytoken;
+  int yytoken = 0;
   /* The variables used to return semantic value and location from the
      action routines.  */
   YYSTYPE yyval;
@@ -1422,7 +1432,6 @@ yyparse ()
      Keep to zero when no symbol should be popped.  */
   int yylen = 0;
 
-  yytoken = 0;
   yyss = yyssa;
   yyvs = yyvsa;
   yystacksize = YYINITDEPTH;
@@ -1440,6 +1449,8 @@ yyparse ()
      The wasted elements are never initialized.  */
   yyssp = yyss;
   yyvsp = yyvs;
+
+  YYLVAL_INITIALIZE ();
   goto yysetstate;
 
 /*------------------------------------------------------------.
@@ -1580,7 +1591,9 @@ yybackup:
   yychar = YYEMPTY;
 
   yystate = yyn;
+  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
   *++yyvsp = yylval;
+  YY_IGNORE_MAYBE_UNINITIALIZED_END
 
   goto yynewstate;
 
@@ -1617,144 +1630,144 @@ yyreduce:
   switch (yyn)
     {
         case 3:
-/* Line 1787 of yacc.c  */
-#line 123 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 124 "grammar.y"
     { cnfDoObj((yyvsp[(2) - (2)].obj)); }
     break;
 
   case 4:
-/* Line 1787 of yacc.c  */
-#line 124 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 125 "grammar.y"
     { cnfDoScript((yyvsp[(2) - (2)].stmt)); }
     break;
 
   case 5:
-/* Line 1787 of yacc.c  */
-#line 125 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 126 "grammar.y"
     { cnfDoCfsysline((yyvsp[(2) - (2)].s)); }
     break;
 
   case 6:
-/* Line 1787 of yacc.c  */
-#line 126 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 127 "grammar.y"
     { cnfDoBSDTag((yyvsp[(2) - (2)].s)); }
     break;
 
   case 7:
-/* Line 1787 of yacc.c  */
-#line 127 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 128 "grammar.y"
     { cnfDoBSDHost((yyvsp[(2) - (2)].s)); }
     break;
 
   case 8:
-/* Line 1787 of yacc.c  */
-#line 128 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 129 "grammar.y"
     { (yyval.obj) = cnfobjNew((yyvsp[(1) - (3)].objType), (yyvsp[(2) - (3)].nvlst)); }
     break;
 
   case 9:
-/* Line 1787 of yacc.c  */
-#line 129 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 130 "grammar.y"
     { (yyval.obj) = cnfobjNew(CNFOBJ_TPL, (yyvsp[(2) - (3)].nvlst)); }
     break;
 
   case 10:
-/* Line 1787 of yacc.c  */
-#line 131 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 132 "grammar.y"
     { (yyval.obj) = cnfobjNew(CNFOBJ_TPL, (yyvsp[(2) - (6)].nvlst));
 					  (yyval.obj)->subobjs = (yyvsp[(5) - (6)].objlst);
 					}
     break;
 
   case 11:
-/* Line 1787 of yacc.c  */
-#line 135 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 136 "grammar.y"
     { (yyval.obj) = cnfobjNew(CNFOBJ_RULESET, (yyvsp[(2) - (6)].nvlst));
 					  (yyval.obj)->script = (yyvsp[(5) - (6)].stmt);
 					}
     break;
 
   case 12:
-/* Line 1787 of yacc.c  */
-#line 138 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 139 "grammar.y"
     { (yyval.objlst) = NULL; }
     break;
 
   case 13:
-/* Line 1787 of yacc.c  */
-#line 139 "grammar.y"
-    { (yyval.objlst) = objlstAdd((yyvsp[(1) - (2)].objlst), (yyvsp[(2) - (2)].obj)); }
-    break;
-
-  case 14:
-/* Line 1787 of yacc.c  */
+/* Line 1813 of yacc.c  */
 #line 140 "grammar.y"
     { (yyval.objlst) = objlstAdd((yyvsp[(1) - (2)].objlst), (yyvsp[(2) - (2)].obj)); }
     break;
 
-  case 15:
-/* Line 1787 of yacc.c  */
+  case 14:
+/* Line 1813 of yacc.c  */
 #line 141 "grammar.y"
+    { (yyval.objlst) = objlstAdd((yyvsp[(1) - (2)].objlst), (yyvsp[(2) - (2)].obj)); }
+    break;
+
+  case 15:
+/* Line 1813 of yacc.c  */
+#line 142 "grammar.y"
     { (yyval.obj) = cnfobjNew(CNFOBJ_PROPERTY, (yyvsp[(2) - (3)].nvlst)); }
     break;
 
   case 16:
-/* Line 1787 of yacc.c  */
-#line 142 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 143 "grammar.y"
     { (yyval.obj) = cnfobjNew(CNFOBJ_CONSTANT, (yyvsp[(2) - (3)].nvlst)); }
     break;
 
   case 17:
-/* Line 1787 of yacc.c  */
-#line 143 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 144 "grammar.y"
     { (yyval.nvlst) = NULL; }
     break;
 
   case 18:
-/* Line 1787 of yacc.c  */
-#line 144 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 145 "grammar.y"
     { (yyvsp[(2) - (2)].nvlst)->next = (yyvsp[(1) - (2)].nvlst); (yyval.nvlst) = (yyvsp[(2) - (2)].nvlst); }
     break;
 
   case 19:
-/* Line 1787 of yacc.c  */
-#line 145 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 146 "grammar.y"
     { (yyval.nvlst) = nvlstSetName((yyvsp[(3) - (3)].nvlst), (yyvsp[(1) - (3)].estr)); }
     break;
 
   case 20:
-/* Line 1787 of yacc.c  */
-#line 146 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 147 "grammar.y"
     { (yyval.nvlst) = nvlstNewStr((yyvsp[(1) - (1)].estr)); }
     break;
 
   case 21:
-/* Line 1787 of yacc.c  */
-#line 147 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 148 "grammar.y"
     { (yyval.nvlst) = nvlstNewArray((yyvsp[(1) - (1)].arr)); }
     break;
 
   case 22:
-/* Line 1787 of yacc.c  */
-#line 148 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 149 "grammar.y"
     { (yyval.stmt) = (yyvsp[(1) - (1)].stmt); }
     break;
 
   case 23:
-/* Line 1787 of yacc.c  */
-#line 149 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 150 "grammar.y"
     { (yyval.stmt) = scriptAddStmt((yyvsp[(1) - (2)].stmt), (yyvsp[(2) - (2)].stmt)); }
     break;
 
   case 24:
-/* Line 1787 of yacc.c  */
-#line 150 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 151 "grammar.y"
     { (yyval.stmt) = (yyvsp[(1) - (1)].stmt); }
     break;
 
   case 25:
-/* Line 1787 of yacc.c  */
-#line 151 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 152 "grammar.y"
     { (yyval.stmt) = cnfstmtNew(S_IF);
 					  (yyval.stmt)->d.s_if.expr = (yyvsp[(2) - (4)].expr);
 					  (yyval.stmt)->d.s_if.t_then = (yyvsp[(4) - (4)].stmt);
@@ -1762,8 +1775,8 @@ yyreduce:
     break;
 
   case 26:
-/* Line 1787 of yacc.c  */
-#line 155 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 156 "grammar.y"
     { (yyval.stmt) = cnfstmtNew(S_IF);
 					  (yyval.stmt)->d.s_if.expr = (yyvsp[(2) - (6)].expr);
 					  (yyval.stmt)->d.s_if.t_then = (yyvsp[(4) - (6)].stmt);
@@ -1771,278 +1784,278 @@ yyreduce:
     break;
 
   case 27:
-/* Line 1787 of yacc.c  */
-#line 159 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 160 "grammar.y"
     { (yyval.stmt) = cnfstmtNewSet((yyvsp[(2) - (5)].s), (yyvsp[(4) - (5)].expr)); }
     break;
 
   case 28:
-/* Line 1787 of yacc.c  */
-#line 160 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 161 "grammar.y"
     { (yyval.stmt) = cnfstmtNewUnset((yyvsp[(2) - (3)].s)); }
     break;
 
   case 29:
-/* Line 1787 of yacc.c  */
-#line 161 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 162 "grammar.y"
     { (yyval.stmt) = cnfstmtNewPRIFILT((yyvsp[(1) - (2)].s), (yyvsp[(2) - (2)].stmt)); }
     break;
 
   case 30:
-/* Line 1787 of yacc.c  */
-#line 162 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 163 "grammar.y"
     { (yyval.stmt) = cnfstmtNewPROPFILT((yyvsp[(1) - (2)].s), (yyvsp[(2) - (2)].stmt)); }
     break;
 
   case 31:
-/* Line 1787 of yacc.c  */
-#line 163 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 164 "grammar.y"
     { (yyval.stmt) = (yyvsp[(1) - (1)].stmt); }
     break;
 
   case 32:
-/* Line 1787 of yacc.c  */
-#line 164 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 165 "grammar.y"
     { (yyval.stmt) = (yyvsp[(2) - (3)].stmt); }
     break;
 
   case 33:
-/* Line 1787 of yacc.c  */
-#line 165 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 166 "grammar.y"
     { (yyval.stmt) = (yyvsp[(1) - (1)].stmt); }
     break;
 
   case 34:
-/* Line 1787 of yacc.c  */
-#line 166 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 167 "grammar.y"
     { (yyval.stmt) = scriptAddStmt((yyvsp[(1) - (3)].stmt), (yyvsp[(3) - (3)].stmt)); }
     break;
 
   case 35:
-/* Line 1787 of yacc.c  */
-#line 168 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 169 "grammar.y"
     { (yyval.stmt) = cnfstmtNewAct((yyvsp[(2) - (3)].nvlst)); }
     break;
 
   case 36:
-/* Line 1787 of yacc.c  */
-#line 169 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 170 "grammar.y"
     { (yyval.stmt) = cnfstmtNewLegaAct((yyvsp[(1) - (1)].s)); }
     break;
 
   case 37:
-/* Line 1787 of yacc.c  */
-#line 170 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 171 "grammar.y"
     { (yyval.stmt) = cnfstmtNew(S_STOP); }
     break;
 
   case 38:
-/* Line 1787 of yacc.c  */
-#line 171 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 172 "grammar.y"
     { (yyval.stmt) = cnfstmtNewCall((yyvsp[(2) - (2)].estr)); }
     break;
 
   case 39:
-/* Line 1787 of yacc.c  */
-#line 172 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 173 "grammar.y"
     { (yyval.stmt) = cnfstmtNewContinue(); }
     break;
 
   case 40:
-/* Line 1787 of yacc.c  */
-#line 173 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 174 "grammar.y"
     { (yyval.expr) = cnfexprNew(AND, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); }
     break;
 
   case 41:
-/* Line 1787 of yacc.c  */
-#line 174 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 175 "grammar.y"
     { (yyval.expr) = cnfexprNew(OR, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); }
     break;
 
   case 42:
-/* Line 1787 of yacc.c  */
-#line 175 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 176 "grammar.y"
     { (yyval.expr) = cnfexprNew(NOT, NULL, (yyvsp[(2) - (2)].expr)); }
     break;
 
   case 43:
-/* Line 1787 of yacc.c  */
-#line 176 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 177 "grammar.y"
     { (yyval.expr) = cnfexprNew(CMP_EQ, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); }
     break;
 
   case 44:
-/* Line 1787 of yacc.c  */
-#line 177 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 178 "grammar.y"
     { (yyval.expr) = cnfexprNew(CMP_NE, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); }
     break;
 
   case 45:
-/* Line 1787 of yacc.c  */
-#line 178 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 179 "grammar.y"
     { (yyval.expr) = cnfexprNew(CMP_LE, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); }
     break;
 
   case 46:
-/* Line 1787 of yacc.c  */
-#line 179 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 180 "grammar.y"
     { (yyval.expr) = cnfexprNew(CMP_GE, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); }
     break;
 
   case 47:
-/* Line 1787 of yacc.c  */
-#line 180 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 181 "grammar.y"
     { (yyval.expr) = cnfexprNew(CMP_LT, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); }
     break;
 
   case 48:
-/* Line 1787 of yacc.c  */
-#line 181 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 182 "grammar.y"
     { (yyval.expr) = cnfexprNew(CMP_GT, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); }
     break;
 
   case 49:
-/* Line 1787 of yacc.c  */
-#line 182 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 183 "grammar.y"
     { (yyval.expr) = cnfexprNew(CMP_CONTAINS, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); }
     break;
 
   case 50:
-/* Line 1787 of yacc.c  */
-#line 183 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 184 "grammar.y"
     { (yyval.expr) = cnfexprNew(CMP_CONTAINSI, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); }
     break;
 
   case 51:
-/* Line 1787 of yacc.c  */
-#line 184 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 185 "grammar.y"
     { (yyval.expr) = cnfexprNew(CMP_STARTSWITH, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); }
     break;
 
   case 52:
-/* Line 1787 of yacc.c  */
-#line 185 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 186 "grammar.y"
     { (yyval.expr) = cnfexprNew(CMP_STARTSWITHI, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); }
     break;
 
   case 53:
-/* Line 1787 of yacc.c  */
-#line 186 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 187 "grammar.y"
     { (yyval.expr) = cnfexprNew('&', (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); }
     break;
 
   case 54:
-/* Line 1787 of yacc.c  */
-#line 187 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 188 "grammar.y"
     { (yyval.expr) = cnfexprNew('+', (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); }
     break;
 
   case 55:
-/* Line 1787 of yacc.c  */
-#line 188 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 189 "grammar.y"
     { (yyval.expr) = cnfexprNew('-', (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); }
     break;
 
   case 56:
-/* Line 1787 of yacc.c  */
-#line 189 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 190 "grammar.y"
     { (yyval.expr) = cnfexprNew('*', (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); }
     break;
 
   case 57:
-/* Line 1787 of yacc.c  */
-#line 190 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 191 "grammar.y"
     { (yyval.expr) = cnfexprNew('/', (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); }
     break;
 
   case 58:
-/* Line 1787 of yacc.c  */
-#line 191 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 192 "grammar.y"
     { (yyval.expr) = cnfexprNew('%', (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); }
     break;
 
   case 59:
-/* Line 1787 of yacc.c  */
-#line 192 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 193 "grammar.y"
     { (yyval.expr) = (yyvsp[(2) - (3)].expr); }
     break;
 
   case 60:
-/* Line 1787 of yacc.c  */
-#line 193 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 194 "grammar.y"
     { (yyval.expr) = cnfexprNew('M', NULL, (yyvsp[(2) - (2)].expr)); }
     break;
 
   case 61:
-/* Line 1787 of yacc.c  */
-#line 194 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 195 "grammar.y"
     { (yyval.expr) = (struct cnfexpr*) cnffuncNew((yyvsp[(1) - (3)].estr), NULL); }
     break;
 
   case 62:
-/* Line 1787 of yacc.c  */
-#line 195 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 196 "grammar.y"
     { (yyval.expr) = (struct cnfexpr*) cnffuncNew((yyvsp[(1) - (4)].estr), (yyvsp[(3) - (4)].fparams)); }
     break;
 
   case 63:
-/* Line 1787 of yacc.c  */
-#line 196 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 197 "grammar.y"
     { (yyval.expr) = (struct cnfexpr*) cnfnumvalNew((yyvsp[(1) - (1)].n)); }
     break;
 
   case 64:
-/* Line 1787 of yacc.c  */
-#line 197 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 198 "grammar.y"
     { (yyval.expr) = (struct cnfexpr*) cnfstringvalNew((yyvsp[(1) - (1)].estr)); }
     break;
 
   case 65:
-/* Line 1787 of yacc.c  */
-#line 198 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 199 "grammar.y"
     { (yyval.expr) = (struct cnfexpr*) cnfvarNew((yyvsp[(1) - (1)].s)); }
     break;
 
   case 66:
-/* Line 1787 of yacc.c  */
-#line 199 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 200 "grammar.y"
     { (yyval.expr) = (struct cnfexpr*) (yyvsp[(1) - (1)].arr); }
     break;
 
   case 67:
-/* Line 1787 of yacc.c  */
-#line 200 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 201 "grammar.y"
     { (yyval.fparams) = cnffparamlstNew((yyvsp[(1) - (1)].expr), NULL); }
     break;
 
   case 68:
-/* Line 1787 of yacc.c  */
-#line 201 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 202 "grammar.y"
     { (yyval.fparams) = cnffparamlstNew((yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].fparams)); }
     break;
 
   case 69:
-/* Line 1787 of yacc.c  */
-#line 202 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 203 "grammar.y"
     { (yyval.arr) = (yyvsp[(2) - (3)].arr); }
     break;
 
   case 70:
-/* Line 1787 of yacc.c  */
-#line 203 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 204 "grammar.y"
     { (yyval.arr) = cnfarrayNew((yyvsp[(1) - (1)].estr)); }
     break;
 
   case 71:
-/* Line 1787 of yacc.c  */
-#line 204 "grammar.y"
+/* Line 1813 of yacc.c  */
+#line 205 "grammar.y"
     { (yyval.arr) = cnfarrayAdd((yyvsp[(1) - (3)].arr), (yyvsp[(3) - (3)].estr)); }
     break;
 
 
-/* Line 1787 of yacc.c  */
-#line 2046 "grammar.c"
+/* Line 1813 of yacc.c  */
+#line 2059 "grammar.c"
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -2205,7 +2218,9 @@ yyerrlab1:
       YY_STACK_PRINT (yyss, yyssp);
     }
 
+  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
   *++yyvsp = yylval;
+  YY_IGNORE_MAYBE_UNINITIALIZED_END
 
 
   /* Shift the error token.  */
@@ -2271,8 +2286,8 @@ yyreturn:
 }
 
 
-/* Line 2048 of yacc.c  */
-#line 206 "grammar.y"
+/* Line 2076 of yacc.c  */
+#line 207 "grammar.y"
 
 /*
 int yyerror(char *s)
@@ -2281,4 +2296,3 @@ int yyerror(char *s)
 	return 0;
 }
 */
-
