@@ -518,12 +518,10 @@ rsRetVal cflineDoAction(rsconf_t *conf, uchar **p, action_t **ppAction)
 			bHadWarning = 1;
 			iRet = RS_RET_OK;
 		}
-		if(iRet == RS_RET_OK || iRet == RS_RET_SUSPENDED) {
-			if((iRet = addAction(&pAction, pMod, pModData, pOMSR, NULL, NULL,
-					     (iRet == RS_RET_SUSPENDED)? 1 : 0)) == RS_RET_OK) {
+		if(iRet == RS_RET_OK) {
+			if((iRet = addAction(&pAction, pMod, pModData, pOMSR, NULL, NULL)) == RS_RET_OK) {
 				/* here check if the module is compatible with select features
 				 * (currently, we have no such features!) */
-				pAction->eState = ACT_STATE_RDY; /* action is enabled */
 				conf->actions.nbrActions++;	/* one more active action! */
 			}
 			break;
