@@ -3,7 +3,7 @@
  *
  * Begun 2010-11-25 RGerhards
  *
- * Copyright (C) 2005-2013 by Rainer Gerhards and Adiscon GmbH
+ * Copyright (C) 2005-2014 by Rainer Gerhards and Adiscon GmbH
  *
  * This file is part of the rsyslog runtime library.
  *
@@ -26,14 +26,10 @@
 #ifndef INCLUDED_TYPEDEFS_H
 #define INCLUDED_TYPEDEFS_H
 #include <stdint.h>
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) || !defined(HAVE_LSEEK64)
 #include <sys/types.h>
 #endif
 
-#ifndef HAVE_LSEEK64
-#include <unistd.h>
-#endif
-
 /* some universal fixed size integer defines ... */
 typedef long long int64;
 typedef long long unsigned uint64;
@@ -113,6 +109,7 @@ typedef uint64 qDeqID;	/* queue Dequeue order ID. 32 bits is considered dangerou
 
 typedef struct tcpLstnPortList_s tcpLstnPortList_t; // TODO: rename?
 typedef struct strmLstnPortList_s strmLstnPortList_t; // TODO: rename?
+typedef struct actWrkrIParams actWrkrIParams_t;
 
 /* under Solaris (actually only SPARC), we need to redefine some types
  * to be void, so that we get void* pointers. Otherwise, we will see
@@ -161,7 +158,9 @@ typedef enum {
 } fiop_t;
 
 #ifndef HAVE_LSEEK64
-	typedef off_t off64_t;
+#	ifndef	HAVE_OFF64_T
+		typedef off_t off64_t;
+#	endif
 #endif
 
 
@@ -194,6 +193,7 @@ typedef uintTiny	propid_t;
 #define PROP_PROCID			21
 #define PROP_MSGID			22
 #define PROP_PARSESUCCESS		23
+#define PROP_JSONMESG			24
 #define PROP_SYS_NOW			150
 #define PROP_SYS_YEAR			151
 #define PROP_SYS_MONTH			152
@@ -269,5 +269,3 @@ struct msgPropDescr_s {
 };
 
 #endif /* multi-include protection */
-/* vim:set ai:
- */
